/*
Google Music Frame

Copyright 2011 Jiří Janoušek <janousek.jiri@gmail.com>

This program is free software: you can redistribute it and/or modify it 
under the terms of the GNU General Public License version 3, as published 
by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranties of 
MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along 
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using Fenryxo.Logger;
using Gtk;
namespace GMF.Helpers{
	
	/**
	 * Tray icon is used to control playback when the main window is not visible
	 * (it's minimized, hidden or covered by other windows) and to bring the hidden main
	 * window to foreground.
	 * 
	 * The tray icon is shown only if Ubuntu Sound menu is not detected.
	 */
	public class TrayIcon:  BaseHelper{
		[Description(nick = "Visibility of the tray icon", blurb = "Is true if a tray icon is shown or Ubuntu Sound menu is present.")]
		public bool visible{ get; private set; default=false; }
		
		[Description(nick = "Availibility of hide action.", blurb = "If true the main window can be hidden to a tray icon or Ubuntu Sound menu.")]
		public bool can_hide{
			get{
#if GLIB_DBUS
				return this.ubuntu_sound_menu.visible || this.icon != null;
#else
				return this.icon != null;
#endif
				
			}
		}
		
		private Menu? menu;
		/* Play/pause menu item */
		private ImageMenuItem? pp_item;
		private StatusIcon? icon;
#if GLIB_DBUS
		private UbuntuSoundMenu ubuntu_sound_menu;
#endif
		
		/**
		 * Creates new tray icon
		 * 
		 * @param app the main application object
		 * @param player playback controls
		 */
		public TrayIcon(GMF.Application app, GMF.Player player){
			base(app, player);
#if GLIB_DBUS
			this.ubuntu_sound_menu = new UbuntuSoundMenu(app);
#endif
		}
		
		/**
		 * Show the tray icon
		 */
		public void show(){
			if(this.visible) return;
#if GLIB_DBUS
			// TODO: option to remove GMF from Ubuntu Sound Menu blacklist
			this.ubuntu_sound_menu.state_changed.connect(this.on_ubuntu_sound_menu_changed);
			this.ubuntu_sound_menu.start();
#else
			this.show_icon();
#endif
			this.visible = true;
		}
		
#if GLIB_DBUS
		/**
		 * Callback when state of the Ubuntu Sound menu is changed
		 */
		private void on_ubuntu_sound_menu_changed(){
			bool visible = this.ubuntu_sound_menu.visible;
			log_debug("Panel icon: Ubuntu Sound menu visible: %s", visible.to_string());
			if(visible)
				this.hide_icon();
			else
				this.show_icon();
		}
#endif
		
		private void show_icon(){
			if(this.icon != null) return;
			this.icon = new StatusIcon.from_icon_name(GMF.Config.APP_ICON);
			this.icon.title = this.app.display_name;
			this.icon.tooltip_text = this.app.display_name;
			this.icon.popup_menu.connect(this.show_menu);
			this.icon.activate.connect(() => {
				var event = Gtk.get_current_event();
					this.show_menu(event.button.button, event.button.time);
			});
			this.create_menu();
		}
		
		/**
		 * Hide the tray icon
		 */
		public void hide(){
			if(!this.visible) return;
#if GLIB_DBUS
			this.ubuntu_sound_menu.state_changed.disconnect(this.on_ubuntu_sound_menu_changed);	
			// TODO: option to disable Ubuntu Sound Menu entry
#endif
			this.hide_icon();
			this.visible = false;
		}
		
		private void hide_icon(){
			if(this.icon == null) return;
			this.icon = null;
			this.remove_menu();
		}
		
		protected override void create_menu(){
			this.menu = new Gtk.Menu();
			ImageMenuItem item;
			item = new ImageMenuItem.with_label(this.app.display_name);
			item.set_image(new Image.from_icon_name(this.app.icon, IconSize.MENU));
			item.activate.connect(() => {
				try{
					this.app.activate();
				}
				catch(Fenryxo.AppError e){
					log_error("Unable to activate: %s", e.message);
				}
			});
			this.menu.append(item);
			
			Actions skip_action = this.player.playback_state == "playing"
				? Actions.PLAY : Actions.PAUSE;
// TODO: report bug, commented code causes wrong C code
//~ 				var actions = Actions.all();
//~ 				foreach(var action in actions){
			for(int a = 0; a < Actions.COUNT; a++){
				// Needed: If "a" is passed directly to this.trigger_action(),
				// it will be always 7 (the last iteration)
				Actions action = (Actions) a;
				if(action == skip_action) continue;
				item = new ImageMenuItem.with_label(action.get_label());
				item.set_image(new Image.from_icon_name(action.get_icon(), IconSize.MENU));
				item.activate.connect(() => {this.trigger_action(action);});
				if(action == Actions.PAUSE || action == Actions.PLAY) this.pp_item = item;
				this.menu.append(item);
			}
		}
		
		private void show_menu(uint button, uint time){
			if(this.menu == null){
				log_critical("TrayIcon: can't show null menu");
				return;
			}
			this.menu.show_all();
			this.menu.popup(null, null, this.icon.position_menu, button, time);
		}
		
		protected override void update_menu(){
			if(this.menu == null) return;
			if(this.player.playback_state == "playing"){
				this.pp_item.label = Actions.PAUSE.get_label();				
				this.pp_item.set_image(new Image.from_icon_name(Actions.PAUSE.get_icon(), IconSize.MENU));
			}
			else{
				this.pp_item.label = Actions.PLAY.get_label();				
				this.pp_item.set_image(new Image.from_icon_name(Actions.PLAY.get_icon(), IconSize.MENU));
			}
		}
		
		protected override void remove_menu(){
			if(this.menu == null) return;
			this.menu.destroy();
			this.menu = null;
			this.pp_item = null;
		}
	}
}
